/*
Winson Chung
Copyright 2003.
For academic use only.
*/

import java.awt.*;
import java.awt.image.*;
import java.util.ArrayList;
import java.io.*;

/**
 * This class does the actual algorithmic comparison between the two images.
 */
public class ComparisonDriver {

  /**
   * Compares the images for differences.
   * @param images the images to be compared.
   * @param offset the offset of pixels to search for differences.
   * @param accuracy the accuracy of the color scale.
   * @return an array of all different sections.
   */
  public static Rectangle[] compareRectangle(BufferedImage[] images, int offset, int accuracy) {
  	ArrayList listBounds = new ArrayList();
	int min_height = Math.min(images[0].getHeight(), images[1].getHeight());
	int min_width = Math.min(images[0].getWidth(), images[1].getWidth());
		try {
  		for (int row = 0;row != min_height;row++) {
  			for (int col = 0;col != min_width;col++) {

					int pixOrig = images[0].getRGB(col, row);
					int pixCopy = images[1].getRGB(col, row);

					if (!ComparisonDriver.withinPixel(images[1], row, col, pixOrig, offset, accuracy)) {
						boolean found = false;
						int loc = 0;
						while (!found && loc != listBounds.size()) {
							Boundary b= (Boundary) listBounds.get(loc);
							if (b.withIn(row, col)) {
								found = true;
								if (b.downByOne(0, row))
									b.set(0, row);
								if (b.downByOne(1, col))
									b.set(1, col);
								if (b.upByOne(2, col))
									b.set(2, col);
								if (b.upByOne(3, row))
									b.set(3, row);
								listBounds.set(loc, b);
							}
							loc++;
						}
						if (!found) {
							listBounds.add(new Boundary(4, row, col, offset));
						}
					}
				}
  		}
		}catch (Exception e) {
			e.printStackTrace();
		}

		Rectangle[] bounds = new Rectangle[listBounds.size()];
		for (int i = 0;i != bounds.length;i++) {
			bounds[i] = ((Boundary) listBounds.get(i)).returnRectangle();
		}
  	return bounds;
  }


  /**
   * Returns whether the pixel is just offsetted by a certain pixel amount.
   * @param image the image to search for the pixels.
   * @param row the row of the original pixel.
   * @param col the col of the original pixel.
   * @param pixOrig the original pixel.
   * @param offset the offset difference.
   * @param accuracy the accuracy of the color difference.
   * @return whether the pixel is offsetted or not.
   */
  public static boolean withinPixel(BufferedImage imageCopy, int row, int col, int pixOrig, int offset, int accuracy) {
  	boolean found = false;
		for (int r = -1 * offset;r<= offset;r++) {
			for (int c = -1 * offset;c<= offset;c++) {
				try {
					int copy = imageCopy.getRGB(col + c, row + r);
					if (ComparisonDriver.withinColor(pixOrig, copy, accuracy)) {
						found = true;
					}
				}catch (Exception e) {}
			}
		}
		return found;
  }

  /**
   * Returns whether the compared pixel is within the accuracy of the original pixel.
   * @param pixOrig the original pixel
   * @param pixCopy the comparison pixel
   * @param accuracy the percentage for which you want the pixels to be within
   * @return whether the pixel is within the rgb range of accuracy
   */
  public static boolean withinColor(int pixOrig, int pixCopy, double accuracy) {
  	double acc = accuracy / 100;
  	boolean same = false;
  	Color origColor = new Color(pixOrig);
  	Color copyColor = new Color(pixCopy);
  	double min = (1-acc);
  	double max = (1+acc);

  	boolean withinRed = (copyColor.getRed() >= (min * origColor.getRed()) && copyColor.getRed() <= (max * origColor.getRed()));
	boolean withinBlue = (copyColor.getBlue() >= (min * origColor.getBlue()) && copyColor.getBlue() <= (max * origColor.getBlue()));
	boolean withinGreen = (copyColor.getGreen() >= (min * origColor.getGreen()) && copyColor.getGreen() <= (max * origColor.getGreen()));

	if (origColor.getRed() >= origColor.getBlue() && origColor.getRed() >= origColor.getGreen())
		same = withinRed;
	else if (origColor.getBlue() >= origColor.getRed() && origColor.getBlue() >= origColor.getGreen())
		same = withinBlue;
	else if (origColor.getGreen() >= origColor.getBlue() && origColor.getGreen() >= origColor.getRed())
		same = withinGreen;

  	return same;
  }
}